import urllib.parse
from typing import Dict
from uuid import uuid4

class MoxfieldDeckAPI:

    def create(self, name: str = None, deck: Dict[str, int] = None, format='none', visibility: str = 'public'):
        if name is None:
            name = str(uuid4())
        if deck is None:
            deck = {}
        payload = {
            'name': name,
            'format': format,
            'visibility': visibility,
            'importText': '\n'.join(f'{val} {key}' for key, val in deck.items()),
            'playStyle': 'paperDollars',
            'pricingProvider': 'tcgplayer'
        }
        headers = {
            'Accept': 'application/json, text/plain, */*',
            'Accept-Language': 'en-US,en;q=0.5',
            'Content-Type': 'application/json'
        }
        r = self.session.post(url="https://api.moxfield.com/v2/decks", json=payload, headers=headers)
        r.raise_for_status()
        resp = r.json()
        return MoxfieldSpecificDeckAPI(resp['publicId'], self.session)

    def __getitem__(self, public_id):
        return MoxfieldSpecificDeckAPI(public_id, self.session)

    def __init__(self, session):
        self.session = session


class MoxfieldSpecificDeckAPI:

    def bulk_edit(self, mainboard, sideboard, maybeboard):
        headers = {
            'Accept': 'application/json, text/plain, */*',
            'Accept-Language': 'en-US,en;q=0.5',
        }
        json = {
            "mainboard":'\n'.join(f'{value} {key}' for key, value in mainboard.items()),
            "sideboard":'\n'.join(f'{value} {key}' for key, value in sideboard.items()),
            "maybeboard":'\n'.join(f'{value} {key}' for key, value in maybeboard.items()),
            "playStyle":"paperDollars",
            "pricingProvider":"tcgplayer"
        }
        url = f'https://api.moxfield.com/v2/decks/{self.public_id}/bulk-edit'
        r = self.session.put(url, json=json, headers=headers)
        return r.json()

    def get(self):
        public_id = urllib.parse.quote(self.public_id, safe='')
        r = self.session.get(
            f'https://api.moxfield.com/v2/decks/all/{public_id}')
        
        r.raise_for_status()

        return r.json()

    def delete(self):
        deck = self.get()
        deck_internal_id = deck['id']
        headers = {
            'Accept': 'application/json, text/plain, */*',
            'Accept-Language': 'en-US,en;q=0.5',
        }
        url = f'https://api.moxfield.com/v1/decks/{deck_internal_id}'
        r = self.session.delete(url, headers=headers)
        r.raise_for_status()

    def __init__(self, public_id, session):
        self.public_id = urllib.parse.quote(public_id, safe='')
        self.mainboard = MoxfieldSpecificDeckCardsAPI(self, session, 'mainboard')
        self.sideboard = MoxfieldSpecificDeckCardsAPI(self, session, 'sideboard')
        self.maybeboard = MoxfieldSpecificDeckCardsAPI(self, session, 'maybeboard')
        self.session = session
        pass

class MoxfieldSpecificDeckCardsAPI:

    def set(self, card_id, card_amount):
        deck = self.deck.get()
        deck_id = deck['id']
        deck_board = deck[self.board]

        value = 0
        for obj in deck_board.values():
            if obj['card']['id'] == card_id:
                value = obj['quantity']

        # No change in card value
        if value == card_amount:
            return

        json = {
            'cardId': card_id, 
            'quantity': card_amount
        }

        headers = {
            'Accept': '*/*',
            'Accept-Language': 'en-US,en;q=0.5',
        }

        url_card_id = urllib.parse.quote(card_id, safe='')

        r = None
        if value == 0:
            url = f'https://api.moxfield.com/v2/decks/{deck_id}/cards/{self.board}'
            r = self.session.post(url, json=json, headers=headers)
        elif card_amount == 0:
            url = f'https://api.moxfield.com/v2/decks/{deck_id}/cards/{self.board}/{url_card_id}'
            r = self.session.delete(url, headers=headers)
        else:
            url = f'https://api.moxfield.com/v2/decks/{deck_id}/cards/{self.board}/{url_card_id}'
            r = self.session.put(url, json=json, headers=headers)
        
        r.raise_for_status()
        return r

    def __init__(self, deck, session, board):
        self.deck = deck
        self.session = session
        self.board = board
        pass


# https://api.moxfield.com/v1/deck-folders/{folder-id}/decks/{deck-internal} POST (move) DELETE (remove)
# https://api.moxfield.com/v1/deck-folders/{folder-id}/ DELETE
# https://api.moxfield.com/v1/deck-folders?pageNumber=n&pageSize=n GET POST (no query params)
import urllib.parse

class MoxfieldDeckFoldersAPI:

    def create(self, name: str):
        payload = {"name": name}
        headers = {
            'Accept': 'application/json, text/plain, */*',
            'Accept-Language': 'en-US,en;q=0.5',
            'Content-Type': 'application/json'
        }
        r = self.session.post('https://api.moxfield.com/v1/deck-folders', json=payload, headers=headers)
        r.raise_for_status()
        resp = r.json()
        return MoxfieldSpecificDeckFolderAPI(resp['id'], self.session)

    def get(self):
        pageNum = 1
        params = {"pageNumber": pageNum, "pageSize": 100}
        headers = {
            'Accept': 'application/json, text/plain, */*',
            'Accept-Language': 'en-US,en;q=0.5'
        }
        r = self.session.get('https://api.moxfield.com/v1/deck-folders', params=params, headers=headers)
        r.raise_for_status()
        resp = r.json()

        for pageNumber in range(2, resp['totalPages'] + 1):
            for obj in resp['data']:
                yield MoxfieldSpecificDeckFolderAPI(obj['id'], self.session)
            
            params['pageNumber'] = pageNumber
            r = self.session.get(f'https://api.moxfield.com/v2/decks/search', params=params, headers=headers)
            r.raise_for_status()
            resp = r.json()

    def __init__(self, session):
        self.session = session

class MoxfieldSpecificDeckFolderAPI:
    
    def remove_deck(self, specific_deck):
        deck = specific_deck.get()
        deck_id = deck['id']
        headers = {
            'Accept': 'application/json, text/plain, */*',
            'Accept-Language': 'en-US,en;q=0.5'
        }
        r = self.session.delete(f'https://api.moxfield.com/v1/deck-folders/{self.folder_id}/decks/{deck_id}', headers=headers)
        r.raise_for_status()

    def add_deck(self, specific_deck):
        deck = specific_deck.get()
        deck_id = deck['id']
        headers = {
            'Accept': 'application/json, text/plain, */*',
            'Accept-Language': 'en-US,en;q=0.5'
        }
        r = self.session.post(f'https://api.moxfield.com/v1/deck-folders/{self.folder_id}/decks/{deck_id}', headers=headers)
        r.raise_for_status()

    def delete(self):
        headers = {
            'Accept': 'application/json, text/plain, */*',
            'Accept-Language': 'en-US,en;q=0.5'
        }
        r = self.session.delete(f'https://api.moxfield.com/v1/deck-folders/{self.folder_id}', headers=headers)
        r.raise_for_status()

    def __init__(self, folder_id, session):
        self.folder_id = urllib.parse.quote(folder_id)
        self.session = session


import dateutil.parser
import requests

from datetime import datetime, timezone

class MoxfieldAuth(requests.auth.AuthBase):

    def __login(self):
        payload = {
            'userName': self.__username, 
            'password': self.__password
        }

        headers = {
            'Accept': 'application/json, text/plain, */*',
            'Accept-Language': 'en-US,en;q=0.5',
            'Content-Type': 'application/json'
        }

        r = requests.post('https://api.moxfield.com/v1/account/token', json=payload, headers=headers)
        r.raise_for_status()

        resp = r.json()
        self.__update(resp)

    def __refresh(self):
        payload = {
            'refreshToken': self.refresh_token
        }

        headers = {
            'Accept': 'application/json, text/plain, */*',
            'Accept-Language': 'en-US,en;q=0.5',
            'Content-Type': 'application/json'
        }

        r = requests.post('https://api.moxfield.com/v1/account/token/refresh', json=payload, headers=headers)

        if r.status_code != 200:
            self.__login()

        resp = r.json()
        self.__update(resp)

    def __update(self, ref_json):
        self.access_token = ref_json['access_token']
        self.refresh_token = ref_json['refresh_token']
        self.expiration = dateutil.parser.parse(ref_json['expiration'])

    def __init__(self, username: str, password: str):
        self.access_token = None
        self.refresh_token = None
        self.expiration = None
        self.__username = username
        self.__password = password

    def __call__(self, r):
        if self.expiration is None:
            self.__login()

        elif datetime.now(timezone.utc) > self.expiration:
            self.__refresh()

        r.headers['Authorization'] = f'Bearer {self.access_token}'
        r.headers['Origin'] = 'https://www.moxfield.com'
        r.headers['Referrer'] = 'https://www.moxfield.com'
        return r